/****************************************************************************
 *
 * parallel_brute_force.c
 *
 * Parallel brute-force matching
 *
 * Copyright (C) 2013, 2016, 2017 Moreno Marzolla
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#include "parallel_brute_force.h"
#include "interval.h"

/**
 * Brute force approach: test each update against each subscription.
 * Returns the number of (upd, subs) matches.
 */
size_t parallel_brute_force( const struct interval* sub, size_t n,
                             const struct interval* upd, size_t m )
{
    size_t result = 0;
    size_t i, j;

#pragma omp parallel for collapse(2) default(none) shared(n, m, sub, upd) reduction(+:result) 
    for (i=0; i<n; ++i) {
	for (j=0; j<m; ++j) {
	    if ( intersect( &sub[i], &upd[j] ) ) {
		result++;
	    }
	}
    }

    return result;
}
